@@ -18,6 +18,7 @@
 #include "mozilla/gfx/Tools.h"
 #include "mozilla/gfx/ssse3-scaler.h"
 #include "mozilla/layers/ImageDataSerializer.h"
+#include "mozilla/layers/NativeLayer.h"
 #include "mozilla/SSE.h"
 #include "mozilla/StaticPrefs_layers.h"
 #include "mozilla/StaticPrefs_nglayout.h"
@@ -26,6 +27,9 @@
 #include "YCbCrUtils.h"
 #include <algorithm>
 #include "ImageContainer.h"
+#ifdef MOZ_WIDGET_GTK
+#  include "mozilla/WidgetUtilsGtk.h"
+#endif
 
 namespace mozilla {
 using namespace mozilla::gfx;
@@ -62,7 +66,10 @@ class DataTextureSourceBasic : public DataTextureSource,
 
   bool Update(gfx::DataSourceSurface* aSurface,
               nsIntRegion* aDestRegion = nullptr,
-              gfx::IntPoint* aSrcOffset = nullptr) override {
+              gfx::IntPoint* aSrcOffset = nullptr,
+              gfx::IntPoint* aDstOffset = nullptr) override {
+    // We can't handle any dst offset
+    MOZ_RELEASE_ASSERT(aDstOffset == nullptr);
     MOZ_ASSERT(!mWrappingExistingData);
     if (mWrappingExistingData) {
       return false;
@@ -141,7 +148,8 @@ class WrappingTextureSourceYCbCrBasic : public DataTextureSource,
 
   virtual bool Update(gfx::DataSourceSurface* aSurface,
                       nsIntRegion* aDestRegion = nullptr,
-                      gfx::IntPoint* aSrcOffset = nullptr) override {
+                      gfx::IntPoint* aSrcOffset = nullptr,
+                      gfx::IntPoint* aDstOffset = nullptr) override {
     return false;
   }
 
@@ -583,6 +591,15 @@ static bool AttemptVideoConvertAndScale(
             ptrdiff_t(dstRect.Y()) * dstStride,
         dstStride);
     aDest->ReleaseBits(dstData);
+#  ifdef MOZ_WIDGET_GTK
+    if (mozilla::widget::IsMainWindowTransparent()) {
+      gfx::Rect rect(dstRect.X(), dstRect.Y(), dstRect.Width(),
+                     dstRect.Height());
+      aDest->FillRect(rect, ColorPattern(DeviceColor(0, 0, 0, 1)),
+                      DrawOptions(1.f, CompositionOp::OP_ADD));
+      aDest->Flush();
+    }
+#  endif
     return true;
   } else
 #endif  // MOZILLA_SSE_HAVE_CPUID_DETECTION
@@ -666,6 +683,10 @@ void BasicCompositor::DrawGeometry(
   // offset can be anywhere.
   IntRect clipRectInRenderTargetSpace =
       aClipRect + mRenderTarget->GetClipSpaceOrigin();
+  if (Maybe<IntRect> rtClip = mRenderTarget->GetClipRect()) {
+    clipRectInRenderTargetSpace =
+        clipRectInRenderTargetSpace.Intersect(*rtClip);
+  }
   buffer->PushClipRect(Rect(clipRectInRenderTargetSpace));
   Rect deviceSpaceClipRect(clipRectInRenderTargetSpace - offset);
 
@@ -1022,21 +1043,29 @@ Maybe<gfx::IntRect> BasicCompositor::BeginRenderingToNativeLayer(
     const nsIntRegion& aOpaqueRegion, NativeLayer* aNativeLayer) {
   IntRect rect = aNativeLayer->GetRect();
 
+  // We only support a single invalid rect per native layer. This is a
+  // limitation that's imposed by the AttemptVideo[ConvertAnd]Scale functions,
+  // which require knowing the combined clip in DrawGeometry and can only handle
+  // a single clip rect.
+  IntRect invalidRect;
   if (mShouldInvalidateWindow) {
-    mInvalidRegion = rect;
+    invalidRect = rect;
   } else {
-    mInvalidRegion.And(aInvalidRegion, rect);
-  }
-
-  if (mInvalidRegion.IsEmpty()) {
-    return Nothing();
+    IntRegion invalidRegion;
+    invalidRegion.And(aInvalidRegion, rect);
+    if (invalidRegion.IsEmpty()) {
+      return Nothing();
+    }
+    invalidRect = invalidRegion.GetBounds();
   }
+  mInvalidRegion = invalidRect;
 
   RefPtr<CompositingRenderTarget> target;
   aNativeLayer->SetSurfaceIsFlipped(false);
-  IntRegion invalidRelativeToLayer = mInvalidRegion.MovedBy(-rect.TopLeft());
+  IntRegion invalidRelativeToLayer = invalidRect - rect.TopLeft();
   RefPtr<DrawTarget> dt = aNativeLayer->NextSurfaceAsDrawTarget(
-      invalidRelativeToLayer, BackendType::SKIA);
+      gfx::IntRect({}, aNativeLayer->GetSize()), invalidRelativeToLayer,
+      BackendType::SKIA);
   if (!dt) {
     return Nothing();
   }
@@ -1049,20 +1078,17 @@ Maybe<gfx::IntRect> BasicCompositor::BeginRenderingToNativeLayer(
   MOZ_RELEASE_ASSERT(target);
   SetRenderTarget(target);
 
-  gfxUtils::ClipToRegion(mRenderTarget->mDrawTarget, mInvalidRegion);
-
-  mRenderTarget->mDrawTarget->PushClipRect(Rect(aClipRect.valueOr(rect)));
+  IntRect clipRect = invalidRect;
+  if (aClipRect) {
+    clipRect = clipRect.Intersect(*aClipRect);
+  }
+  mRenderTarget->SetClipRect(Some(clipRect));
 
   return Some(rect);
 }
 
 void BasicCompositor::EndRenderingToNativeLayer() {
-  // Pop aClipRect/bounds rect
-  mRenderTarget->mDrawTarget->PopClip();
-
-  // Pop mInvalidRegion
-  mRenderTarget->mDrawTarget->PopClip();
-
+  mRenderTarget->SetClipRect(Nothing());
   SetRenderTarget(mNativeLayersReferenceRT);
 
   MOZ_RELEASE_ASSERT(mCurrentNativeLayer);
@@ -1133,7 +1159,7 @@ void BasicCompositor::TryToEndRemoteDrawing() {
     RefPtr<Runnable> runnable =
         NS_NewRunnableFunction("layers::BasicCompositor::TryToEndRemoteDrawing",
                                [self]() { self->TryToEndRemoteDrawing(); });
-    MessageLoop::current()->PostDelayedTask(runnable.forget(), retryMs);
+    GetCurrentSerialEventTarget()->DelayedDispatch(runnable.forget(), retryMs);
   } else {
     EndRemoteDrawing();
   }
