@@ -15,7 +15,8 @@
 #include "mozilla/layers/AnimationHelper.h"  // for GetAnimatedPropValue
 #include "mozilla/layers/CanvasLayerComposite.h"
 #include "mozilla/layers/ColorLayerComposite.h"
-#include "mozilla/layers/Compositor.h"  // for Compositor
+#include "mozilla/layers/Compositor.h"                  // for Compositor
+#include "mozilla/layers/CompositorAnimationStorage.h"  // for CompositorAnimationStorage
 #include "mozilla/layers/ContainerLayerComposite.h"
 #include "mozilla/layers/ImageBridgeParent.h"  // for ImageBridgeParent
 #include "mozilla/layers/ImageLayerComposite.h"
@@ -37,7 +38,8 @@
 #include "nsPoint.h"          // for nsPoint
 #include "nsTArray.h"         // for nsTArray, nsTArray_Impl, etc
 #include "TreeTraversal.h"    // for ForEachNode
-#include "GeckoProfiler.h"
+#include "mozilla/ProfilerLabels.h"
+#include "mozilla/ProfilerMarkers.h"
 #include "mozilla/layers/TextureHost.h"
 #include "mozilla/layers/AsyncCompositionManager.h"
 
@@ -73,8 +75,7 @@ void LayerTransactionParent::SetLayerManager(
     return;
   }
   mLayerManager = aLayerManager;
-  for (auto iter = mLayerMap.Iter(); !iter.Done(); iter.Next()) {
-    auto layer = iter.Data();
+  for (const auto& layer : mLayerMap.Values()) {
     if (mAnimStorage && layer->GetCompositorAnimationsId()) {
       mAnimStorage->ClearById(layer->GetCompositorAnimationsId());
     }
@@ -102,8 +103,7 @@ void LayerTransactionParent::Destroy() {
   }
   mDestroyed = true;
   if (mAnimStorage) {
-    for (auto iter = mLayerMap.Iter(); !iter.Done(); iter.Next()) {
-      auto layer = iter.Data();
+    for (const auto& layer : mLayerMap.Values()) {
       if (layer->GetCompositorAnimationsId()) {
         mAnimStorage->ClearById(layer->GetCompositorAnimationsId());
       }
@@ -523,7 +523,7 @@ bool LayerTransactionParent::SetLayerAttributes(
   } else {
     layer->SetMaskLayer(nullptr);
   }
-  layer->SetCompositorAnimations(common.compositorAnimations());
+  layer->SetCompositorAnimations(mId, common.compositorAnimations());
   // Clean up the Animations by id in the CompositorAnimationStorage
   // if there are no active animations on the layer
   if (mAnimStorage && layer->GetCompositorAnimationsId() &&
@@ -717,14 +717,12 @@ mozilla::ipc::IPCResult LayerTransactionParent::RecvGetTransform(
   }
   float scale = 1;
   Point3D scaledOrigin;
-  Point3D transformOrigin;
   if (layer->GetTransformData()) {
     const TransformData& data = *layer->GetTransformData();
     scale = data.appUnitsPerDevPixel();
     scaledOrigin = Point3D(
         NS_round(NSAppUnitsToFloatPixels(data.origin().x, scale)),
         NS_round(NSAppUnitsToFloatPixels(data.origin().y, scale)), 0.0f);
-    transformOrigin = data.transformOrigin();
   }
 
   // If our parent isn't a perspective layer, then the offset into reference
@@ -773,6 +771,12 @@ mozilla::ipc::IPCResult LayerTransactionParent::RecvGetAPZTestData(
   return IPC_OK();
 }
 
+mozilla::ipc::IPCResult LayerTransactionParent::RecvGetFrameUniformity(
+    FrameUniformityData* aOutData) {
+  mCompositorBridge->GetFrameUniformity(GetId(), aOutData);
+  return IPC_OK();
+}
+
 mozilla::ipc::IPCResult LayerTransactionParent::RecvRequestProperty(
     const nsString& aProperty, float* aValue) {
   *aValue = -1;
@@ -790,7 +794,8 @@ mozilla::ipc::IPCResult LayerTransactionParent::RecvSetConfirmedTargetAPZC(
       return IPC_FAIL(this, "Bad layers id");
     }
   }
-  mCompositorBridge->SetConfirmedTargetAPZC(GetId(), aBlockId, aTargets);
+  mCompositorBridge->SetConfirmedTargetAPZC(GetId(), aBlockId,
+                                            std::move(aTargets));
   return IPC_OK();
 }
 
@@ -883,11 +888,11 @@ void LayerTransactionParent::SetPendingTransactionId(
       aTxnEndTime, aFwdTime, aURL, aContainsSVG});
 }
 
-TransactionId LayerTransactionParent::FlushTransactionId(
-    const VsyncId& aCompositeId, TimeStamp& aCompositeEnd) {
-  TransactionId id;
+void LayerTransactionParent::FlushPendingTransactions(
+    const VsyncId& aCompositeId, TimeStamp& aCompositeEnd,
+    nsTArray<TransactionId>& aOutTransactions) {
   for (auto& transaction : mPendingTransactions) {
-    id = transaction.mId;
+    aOutTransactions.AppendElement(transaction.mId);
     if (mId.IsValid() && transaction.mId.IsValid() && !mVsyncRate.IsZero()) {
       RecordContentFrameTime(
           transaction.mTxnVsyncId, transaction.mVsyncStartTime,
@@ -919,7 +924,6 @@ TransactionId LayerTransactionParent::FlushTransactionId(
   }
 
   mPendingTransactions.Clear();
-  return id;
 }
 
 void LayerTransactionParent::SendAsyncMessage(
@@ -945,12 +949,13 @@ bool LayerTransactionParent::BindLayerToHandle(RefPtr<Layer> aLayer,
   if (!aHandle || !aLayer) {
     return false;
   }
-  if (auto entry = mLayerMap.LookupForAdd(aHandle.Value())) {
-    return false;
-  } else {
-    entry.OrInsert([&aLayer]() { return aLayer; });
-  }
-  return true;
+  return mLayerMap.WithEntryHandle(aHandle.Value(), [&](auto&& entry) {
+    if (entry) {
+      return false;
+    }
+    entry.Insert(std::move(aLayer));
+    return true;
+  });
 }
 
 Layer* LayerTransactionParent::AsLayer(const LayerHandle& aHandle) {
