@@ -7,61 +7,63 @@
 #define PANGO_ENABLE_ENGINE
 
 #include "gfxPlatformGtk.h"
-#include "prenv.h"
 
-#include "nsUnicharUtils.h"
-#include "nsUnicodeProperties.h"
+#include <gtk/gtk.h>
+#include <fontconfig/fontconfig.h>
+
+#include "base/task.h"
+#include "base/thread.h"
+#include "base/message_loop.h"
+#include "cairo.h"
 #include "gfx2DGlue.h"
 #include "gfxFcPlatformFontList.h"
 #include "gfxConfig.h"
 #include "gfxContext.h"
+#include "gfxImageSurface.h"
 #include "gfxUserFontSet.h"
 #include "gfxUtils.h"
 #include "gfxFT2FontBase.h"
 #include "gfxTextRun.h"
-#include "VsyncSource.h"
+#include "GLContextProvider.h"
 #include "mozilla/Atomics.h"
-#include "mozilla/Monitor.h"
-#include "mozilla/StaticPrefs_layers.h"
-#include "base/task.h"
-#include "base/thread.h"
-#include "base/message_loop.h"
+#include "mozilla/Components.h"
+#include "mozilla/dom/ContentChild.h"
 #include "mozilla/FontPropertyTypes.h"
-#include "mozilla/gfx/Logging.h"
-
 #include "mozilla/gfx/2D.h"
+#include "mozilla/gfx/Logging.h"
+#include "mozilla/gfx/XlibDisplay.h"
+#include "mozilla/Monitor.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/StaticPrefs_gfx.h"
+#include "mozilla/StaticPrefs_layers.h"
+#include "nsAppRunner.h"
+#include "nsIGfxInfo.h"
+#include "nsMathUtils.h"
+#include "nsUnicharUtils.h"
+#include "nsUnicodeProperties.h"
+#include "VsyncSource.h"
 
-#include "cairo.h"
-#include <gtk/gtk.h>
-
-#include "gfxImageSurface.h"
 #ifdef MOZ_X11
 #  include <gdk/gdkx.h>
-#  include "gfxXlibSurface.h"
 #  include "cairo-xlib.h"
-#  include "mozilla/Preferences.h"
-#  include "mozilla/X11Util.h"
-
-#  include "GLContextProvider.h"
+#  include "gfxXlibSurface.h"
 #  include "GLContextGLX.h"
 #  include "GLXLibrary.h"
+#  include "mozilla/X11Util.h"
 
 /* Undefine the Status from Xlib since it will conflict with system headers on
  * OSX */
 #  if defined(__APPLE__) && defined(Status)
 #    undef Status
 #  endif
-
-#  ifdef MOZ_WAYLAND
-#    include <gdk/gdkwayland.h>
-#    include "mozilla/widget/nsWaylandDisplay.h"
-#  endif
-
 #endif /* MOZ_X11 */
 
-#include <fontconfig/fontconfig.h>
-
-#include "nsMathUtils.h"
+#ifdef MOZ_WAYLAND
+#  include <gdk/gdkwayland.h>
+#  include "mozilla/widget/nsWaylandDisplay.h"
+#  include "mozilla/widget/DMABufLibWrapper.h"
+#  include "mozilla/StaticPrefs_widget.h"
+#endif
 
 #define GDK_PIXMAP_SIZE_MAX 32767
 
@@ -72,9 +74,23 @@ using namespace mozilla;
 using namespace mozilla::gfx;
 using namespace mozilla::unicode;
 using namespace mozilla::widget;
-using mozilla::dom::SystemFontListEntry;
 
 static FT_Library gPlatformFTLibrary = nullptr;
+static int32_t sDPI;
+
+static void screen_resolution_changed(GdkScreen* aScreen, GParamSpec* aPspec,
+                                      gpointer aClosure) {
+  sDPI = 0;
+}
+
+#if defined(MOZ_X11)
+// TODO(aosmond): The envvar is deprecated. We should remove it once EGL is the
+// default in release.
+static bool IsX11EGLEnvvarEnabled() {
+  const char* eglPref = PR_GetEnv("MOZ_X11_EGL");
+  return (eglPref && *eglPref);
+}
+#endif
 
 gfxPlatformGtk::gfxPlatformGtk() {
   if (!gfxPlatform::IsHeadless()) {
@@ -82,46 +98,161 @@ gfxPlatformGtk::gfxPlatformGtk() {
   }
 
   mMaxGenericSubstitutions = UNINITIALIZED_VALUE;
-  mIsX11Display = gfxPlatform::IsHeadless()
-                      ? false
-                      : GDK_IS_X11_DISPLAY(gdk_display_get_default());
+  mIsX11Display = gfxPlatform::IsHeadless() ? false : GdkIsX11Display();
+  if (XRE_IsParentProcess()) {
 #ifdef MOZ_X11
-  if (mIsX11Display && XRE_IsParentProcess() &&
-      mozilla::Preferences::GetBool("gfx.xrender.enabled")) {
-    gfxVars::SetUseXRender(true);
+    if (mIsX11Display && mozilla::Preferences::GetBool("gfx.xrender.enabled")) {
+      gfxVars::SetUseXRender(true);
+    }
+#endif
+
+    InitX11EGLConfig();
+    if (IsWaylandDisplay() || gfxConfig::IsEnabled(Feature::X11_EGL)) {
+      gfxVars::SetUseEGL(true);
+    }
+
+    InitDmabufConfig();
+    if (gfxConfig::IsEnabled(Feature::DMABUF)) {
+      gfxVars::SetUseDMABuf(true);
+    }
   }
+
+#ifdef MOZ_WAYLAND
+  mUseWebGLDmabufBackend = true;
 #endif
 
   InitBackendPrefs(GetBackendPrefs());
 
-#ifdef MOZ_X11
-  if (mIsX11Display) {
-    mCompositorDisplay = XOpenDisplay(nullptr);
-    MOZ_ASSERT(mCompositorDisplay, "Failed to create compositor display!");
-  } else {
-    mCompositorDisplay = nullptr;
-  }
-#endif  // MOZ_X11
   gPlatformFTLibrary = Factory::NewFTLibrary();
-  MOZ_ASSERT(gPlatformFTLibrary);
+  MOZ_RELEASE_ASSERT(gPlatformFTLibrary);
   Factory::SetFTLibrary(gPlatformFTLibrary);
+
+  GdkScreen* gdkScreen = gdk_screen_get_default();
+  if (gdkScreen) {
+    g_signal_connect(gdkScreen, "notify::resolution",
+                     G_CALLBACK(screen_resolution_changed), nullptr);
+  }
+
+  // Bug 1714483: Force disable FXAA Antialiasing on NV drivers. This is a
+  // temporary workaround for a driver bug.
+  PR_SetEnv("__GL_ALLOW_FXAA_USAGE=0");
 }
 
 gfxPlatformGtk::~gfxPlatformGtk() {
-#ifdef MOZ_X11
-  if (mCompositorDisplay) {
-    XCloseDisplay(mCompositorDisplay);
-  }
-#endif  // MOZ_X11
-
   Factory::ReleaseFTLibrary(gPlatformFTLibrary);
   gPlatformFTLibrary = nullptr;
 }
 
-void gfxPlatformGtk::FlushContentDrawing() {
-  if (gfxVars::UseXRender()) {
-    XFlush(DefaultXDisplay());
+void gfxPlatformGtk::InitX11EGLConfig() {
+  FeatureState& feature = gfxConfig::GetFeature(Feature::X11_EGL);
+#ifdef MOZ_X11
+  feature.EnableByDefault();
+
+  if (StaticPrefs::gfx_x11_egl_force_enabled_AtStartup()) {
+    feature.UserForceEnable("Force enabled by pref");
+  } else if (IsX11EGLEnvvarEnabled()) {
+    feature.UserForceEnable("Force enabled by envvar");
+  } else if (StaticPrefs::gfx_x11_egl_force_disabled_AtStartup()) {
+    feature.UserDisable("Force disabled by pref",
+                        "FEATURE_FAILURE_USER_FORCE_DISABLED"_ns);
   }
+
+  nsCString failureId;
+  int32_t status;
+  nsCOMPtr<nsIGfxInfo> gfxInfo = components::GfxInfo::Service();
+  if (NS_FAILED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_X11_EGL,
+                                          failureId, &status))) {
+    feature.Disable(FeatureStatus::BlockedNoGfxInfo, "gfxInfo is broken",
+                    "FEATURE_FAILURE_NO_GFX_INFO"_ns);
+  } else if (status != nsIGfxInfo::FEATURE_STATUS_OK) {
+    feature.Disable(FeatureStatus::Blocklisted, "Blocklisted by gfxInfo",
+                    failureId);
+  }
+
+  nsAutoString testType;
+  gfxInfo->GetTestType(testType);
+  // We can only use X11/EGL if we actually found the EGL library and
+  // successfully use it to determine system information in glxtest.
+  if (testType != u"EGL") {
+    feature.ForceDisable(FeatureStatus::Broken, "glxtest could not use EGL",
+                         "FEATURE_FAILURE_GLXTEST_NO_EGL"_ns);
+  }
+#else
+  feature.DisableByDefault(FeatureStatus::Unavailable, "X11 support missing",
+                           "FEATURE_FAILURE_NO_X11"_ns);
+#endif
+}
+
+void gfxPlatformGtk::InitDmabufConfig() {
+  FeatureState& feature = gfxConfig::GetFeature(Feature::DMABUF);
+#ifdef MOZ_WAYLAND
+  feature.EnableByDefault();
+
+  if (StaticPrefs::widget_dmabuf_force_enabled_AtStartup()) {
+    feature.UserForceEnable("Force enabled by pref");
+  }
+
+  nsCString failureId;
+  int32_t status;
+  nsCOMPtr<nsIGfxInfo> gfxInfo = components::GfxInfo::Service();
+  if (NS_FAILED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_DMABUF, failureId,
+                                          &status))) {
+    feature.Disable(FeatureStatus::BlockedNoGfxInfo, "gfxInfo is broken",
+                    "FEATURE_FAILURE_NO_GFX_INFO"_ns);
+  } else if (status != nsIGfxInfo::FEATURE_STATUS_OK) {
+    feature.Disable(FeatureStatus::Blocklisted, "Blocklisted by gfxInfo",
+                    failureId);
+  }
+
+  if (!gfxVars::UseEGL()) {
+    feature.ForceDisable(FeatureStatus::Unavailable, "Requires EGL",
+                         "FEATURE_FAILURE_REQUIRES_EGL"_ns);
+  }
+
+  if (feature.IsEnabled()) {
+    nsAutoCString drmRenderDevice;
+    gfxInfo->GetDrmRenderDevice(drmRenderDevice);
+    gfxVars::SetDrmRenderDevice(drmRenderDevice);
+
+    if (!GetDMABufDevice()->Configure(failureId)) {
+      feature.ForceDisable(FeatureStatus::Failed, "Failed to configure",
+                           failureId);
+    }
+  }
+#else
+  feature.DisableByDefault(FeatureStatus::Unavailable,
+                           "Wayland support missing",
+                           "FEATURE_FAILURE_NO_WAYLAND"_ns);
+#endif
+}
+
+void gfxPlatformGtk::InitWebRenderConfig() {
+  gfxPlatform::InitWebRenderConfig();
+
+  if (!XRE_IsParentProcess()) {
+    return;
+  }
+
+  FeatureState& feature = gfxConfig::GetFeature(Feature::WEBRENDER_COMPOSITOR);
+  if (feature.IsEnabled()) {
+    if (!(gfxConfig::IsEnabled(Feature::WEBRENDER) ||
+          gfxConfig::IsEnabled(Feature::WEBRENDER_SOFTWARE))) {
+      feature.ForceDisable(FeatureStatus::Unavailable, "WebRender disabled",
+                           "FEATURE_FAILURE_WR_DISABLED"_ns);
+    } else if (!IsWaylandDisplay()) {
+      feature.ForceDisable(FeatureStatus::Unavailable,
+                           "Wayland support missing",
+                           "FEATURE_FAILURE_NO_WAYLAND"_ns);
+    }
+#ifdef MOZ_WAYLAND
+    else if (!widget::WaylandDisplayGet()->GetViewporter()) {
+      feature.ForceDisable(FeatureStatus::Unavailable,
+                           "Requires wp_viewporter protocol support",
+                           "FEATURE_FAILURE_REQUIRES_WPVIEWPORTER"_ns);
+    }
+#endif
+  }
+  gfxVars::SetUseWebRenderCompositor(feature.IsEnabled());
 }
 
 void gfxPlatformGtk::InitPlatformGPUProcessPrefs() {
@@ -130,7 +261,7 @@ void gfxPlatformGtk::InitPlatformGPUProcessPrefs() {
     FeatureState& gpuProc = gfxConfig::GetFeature(Feature::GPU_PROCESS);
     gpuProc.ForceDisable(FeatureStatus::Blocked,
                          "Wayland does not work in the GPU process",
-                         NS_LITERAL_CSTRING("FEATURE_FAILURE_WAYLAND"));
+                         "FEATURE_FAILURE_WAYLAND"_ns);
   }
 #endif
 }
@@ -196,11 +327,6 @@ nsresult gfxPlatformGtk::GetFontList(nsAtom* aLangGroup,
   return NS_OK;
 }
 
-nsresult gfxPlatformGtk::UpdateFontList() {
-  gfxPlatformFontList::PlatformFontList()->UpdateFontList();
-  return NS_OK;
-}
-
 // xxx - this is ubuntu centric, need to go through other distros and flesh
 // out a more general list
 static const char kFontDejaVuSans[] = "DejaVu Sans";
@@ -213,18 +339,14 @@ static const char kFontDroidSansFallback[] = "Droid Sans Fallback";
 static const char kFontWenQuanYiMicroHei[] = "WenQuanYi Micro Hei";
 static const char kFontNanumGothic[] = "NanumGothic";
 static const char kFontSymbola[] = "Symbola";
+static const char kFontNotoSansSymbols[] = "Noto Sans Symbols";
+static const char kFontNotoSansSymbols2[] = "Noto Sans Symbols2";
 
-void gfxPlatformGtk::GetCommonFallbackFonts(uint32_t aCh, uint32_t aNextCh,
-                                            Script aRunScript,
+void gfxPlatformGtk::GetCommonFallbackFonts(uint32_t aCh, Script aRunScript,
+                                            eFontPresentation aPresentation,
                                             nsTArray<const char*>& aFontList) {
-  EmojiPresentation emoji = GetEmojiPresentation(aCh);
-  if (emoji != EmojiPresentation::TextOnly) {
-    if (aNextCh == kVariationSelector16 ||
-        (aNextCh != kVariationSelector15 &&
-         emoji == EmojiPresentation::EmojiDefault)) {
-      // if char is followed by VS16, try for a color emoji glyph
-      aFontList.AppendElement(kFontTwemojiMozilla);
-    }
+  if (PrefersColor(aPresentation)) {
+    aFontList.AppendElement(kFontTwemojiMozilla);
   }
 
   aFontList.AppendElement(kFontDejaVuSerif);
@@ -232,6 +354,8 @@ void gfxPlatformGtk::GetCommonFallbackFonts(uint32_t aCh, uint32_t aNextCh,
   aFontList.AppendElement(kFontDejaVuSans);
   aFontList.AppendElement(kFontFreeSans);
   aFontList.AppendElement(kFontSymbola);
+  aFontList.AppendElement(kFontNotoSansSymbols);
+  aFontList.AppendElement(kFontNotoSansSymbols2);
 
   // add fonts for CJK ranges
   // xxx - this isn't really correct, should use the same CJK font ordering
@@ -246,35 +370,31 @@ void gfxPlatformGtk::GetCommonFallbackFonts(uint32_t aCh, uint32_t aNextCh,
 }
 
 void gfxPlatformGtk::ReadSystemFontList(
-    nsTArray<SystemFontListEntry>* retValue) {
+    mozilla::dom::SystemFontList* retValue) {
   gfxFcPlatformFontList::PlatformFontList()->ReadSystemFontList(retValue);
 }
 
-gfxPlatformFontList* gfxPlatformGtk::CreatePlatformFontList() {
-  gfxPlatformFontList* list = new gfxFcPlatformFontList();
-  if (NS_SUCCEEDED(list->InitFontList())) {
-    return list;
-  }
-  gfxPlatformFontList::Shutdown();
-  return nullptr;
+bool gfxPlatformGtk::CreatePlatformFontList() {
+  return gfxPlatformFontList::Initialize(new gfxFcPlatformFontList);
 }
 
-// FIXME(emilio, bug 1554850): This should be invalidated somehow, right now
-// requires a restart.
-static int32_t sDPI = 0;
-
 int32_t gfxPlatformGtk::GetFontScaleDPI() {
-  if (MOZ_UNLIKELY(!sDPI)) {
-    // Make sure init is run so we have a resolution
-    GdkScreen* screen = gdk_screen_get_default();
-    gtk_settings_get_for_screen(screen);
-    sDPI = int32_t(round(gdk_screen_get_resolution(screen)));
-    if (sDPI <= 0) {
-      // Fall back to something sane
-      sDPI = 96;
-    }
+  MOZ_ASSERT(XRE_IsParentProcess(),
+             "You can access this via LookAndFeel if you need it in child "
+             "processes");
+  if (MOZ_LIKELY(sDPI != 0)) {
+    return sDPI;
   }
-  return sDPI;
+  GdkScreen* screen = gdk_screen_get_default();
+  // Ensure settings in config files are processed.
+  gtk_settings_get_for_screen(screen);
+  int32_t dpi = int32_t(round(gdk_screen_get_resolution(screen)));
+  if (dpi <= 0) {
+    // Fall back to something sane
+    dpi = 96;
+  }
+  sDPI = dpi;
+  return dpi;
 }
 
 double gfxPlatformGtk::GetFontScaleFactor() {
@@ -319,7 +439,7 @@ gfxImageFormat gfxPlatformGtk::GetOffscreenFormat() {
 
 void gfxPlatformGtk::FontsPrefsChanged(const char* aPref) {
   // only checking for generic substitions, pass other changes up
-  if (strcmp(GFX_PREF_MAX_GENERIC_SUBSTITUTIONS, aPref)) {
+  if (strcmp(GFX_PREF_MAX_GENERIC_SUBSTITUTIONS, aPref) != 0) {
     gfxPlatform::FontsPrefsChanged(aPref);
     return;
   }
@@ -344,6 +464,15 @@ uint32_t gfxPlatformGtk::MaxGenericSubstitions() {
 
 bool gfxPlatformGtk::AccelerateLayersByDefault() { return true; }
 
+#ifdef MOZ_WAYLAND
+bool gfxPlatformGtk::UseDMABufWebGL() {
+  static bool dmabufAvailable = []() {
+    return gfxVars::UseDMABuf() && GetDMABufDevice()->IsDMABufWebGLEnabled();
+  }();
+  return dmabufAvailable && mUseWebGLDmabufBackend;
+}
+#endif
+
 #if defined(MOZ_X11)
 
 static nsTArray<uint8_t> GetDisplayICCProfile(Display* dpy, Window& root) {
@@ -381,6 +510,26 @@ nsTArray<uint8_t> gfxPlatformGtk::GetPlatformCMSOutputProfileData() {
     return prefProfileData;
   }
 
+  if (XRE_IsContentProcess()) {
+    MOZ_ASSERT(NS_IsMainThread());
+    // This will be passed in during InitChild so we can avoid sending a
+    // sync message back to the parent during init.
+    const mozilla::gfx::ContentDeviceData* contentDeviceData =
+        GetInitContentDeviceData();
+    if (contentDeviceData) {
+      // On Windows, we assert that the profile isn't empty, but on
+      // Linux it can legitimately be empty if the display isn't
+      // calibrated.  Thus, no assertion here.
+      return contentDeviceData->cmsOutputProfileData().Clone();
+    }
+
+    // Otherwise we need to ask the parent for the updated color profile
+    mozilla::dom::ContentChild* cc = mozilla::dom::ContentChild::GetSingleton();
+    nsTArray<uint8_t> result;
+    Unused << cc->SendGetOutputColorProfileData(&result);
+    return result;
+  }
+
   if (!mIsX11Display) {
     return nsTArray<uint8_t>();
   }
@@ -425,7 +574,7 @@ nsTArray<uint8_t> gfxPlatformGtk::GetPlatformCMSOutputProfileData() {
   }
 
   // Format documented in "VESA E-EDID Implementation Guide"
-  float gamma = (100 + retProperty[0x17]) / 100.0f;
+  float gamma = (100 + (float)retProperty[0x17]) / 100.0f;
 
   qcms_CIE_xyY whitePoint;
   whitePoint.x =
@@ -466,7 +615,9 @@ nsTArray<uint8_t> gfxPlatformGtk::GetPlatformCMSOutputProfileData() {
   result.AppendElements(static_cast<uint8_t*>(mem), size);
   free(mem);
 
-  return result;
+  // XXX: It seems like we get wrong colors when using this constructed profile:
+  // See bug 1696819. For now just forget that we made it.
+  return nsTArray<uint8_t>();
 }
 
 #else  // defined(MOZ_X11)
@@ -556,8 +707,8 @@ class GtkVsyncSource final : public VsyncSource {
       }
 
       mGLContext = gl::GLContextGLX::CreateGLContext(
-          gl::CreateContextFlags::NONE, gl::SurfaceCaps::Any(), false,
-          mXDisplay, root, config, false, nullptr);
+          {}, gfx::XlibDisplay::Borrow(mXDisplay), root, config, false,
+          nullptr);
 
       if (!mGLContext) {
         lock.NotifyAll();
@@ -644,7 +795,7 @@ class GtkVsyncSource final : public VsyncSource {
         // until the parity of the counter value changes.
         unsigned int nextSync = syncCounter + 1;
         int status;
-        if ((status = gl::sGLXLibrary.fWaitVideoSync(2, nextSync % 2,
+        if ((status = gl::sGLXLibrary.fWaitVideoSync(2, (int)nextSync % 2,
                                                      &syncCounter)) != 0) {
           gfxWarningOnce() << "glXWaitVideoSync returned " << status;
           useSoftware = true;
@@ -660,12 +811,13 @@ class GtkVsyncSource final : public VsyncSource {
           double remaining =
               (1000.f / 60.f) - (TimeStamp::Now() - lastVsync).ToMilliseconds();
           if (remaining > 0) {
-            PlatformThread::Sleep(remaining);
+            PlatformThread::Sleep((int)remaining);
           }
         }
 
         lastVsync = TimeStamp::Now();
-        NotifyVsync(lastVsync);
+        TimeStamp outputTime = lastVsync + GetVsyncRate();
+        NotifyVsync(lastVsync, outputTime);
       }
     }
 
@@ -700,11 +852,25 @@ already_AddRefed<gfx::VsyncSource> gfxPlatformGtk::CreateHardwareVsyncSource() {
   }
 #  endif
 
-  // Only use GLX vsync when the OpenGL compositor is being used.
+  // Only use GLX vsync when the OpenGL compositor / WebRender is being used.
   // The extra cost of initializing a GLX context while blocking the main
   // thread is not worth it when using basic composition.
+  //
+  // Don't call gl::sGLXLibrary.SupportsVideoSync() when EGL is used.
+  // NVIDIA drivers refuse to use EGL GL context when GLX was initialized first
+  // and fail silently.
   if (gfxConfig::IsEnabled(Feature::HW_COMPOSITING)) {
-    if (gl::sGLXLibrary.SupportsVideoSync()) {
+    bool useGlxVsync = false;
+
+    nsCOMPtr<nsIGfxInfo> gfxInfo = components::GfxInfo::Service();
+    nsString adapterDriverVendor;
+    gfxInfo->GetAdapterDriverVendor(adapterDriverVendor);
+
+    // Nvidia doesn't support GLX at the same time as EGL but Mesa does.
+    if (!gfxVars::UseEGL() || (adapterDriverVendor.Find("mesa") != -1)) {
+      useGlxVsync = gl::sGLXLibrary.SupportsVideoSync(DefaultXDisplay());
+    }
+    if (useGlxVsync) {
       RefPtr<VsyncSource> vsyncSource = new GtkVsyncSource();
       VsyncSource::Display& display = vsyncSource->GetGlobalDisplay();
       if (!static_cast<GtkVsyncSource::GLXDisplay&>(display).Setup()) {
@@ -721,15 +887,8 @@ already_AddRefed<gfx::VsyncSource> gfxPlatformGtk::CreateHardwareVsyncSource() {
 
 #endif
 
-#ifdef MOZ_WAYLAND
-bool gfxPlatformGtk::UseWaylandDMABufTextures() {
-  return IsWaylandDisplay() && nsWaylandDisplay::IsDMABufTexturesEnabled();
+void gfxPlatformGtk::BuildContentDeviceData(ContentDeviceData* aOut) {
+  gfxPlatform::BuildContentDeviceData(aOut);
+
+  aOut->cmsOutputProfileData() = GetPlatformCMSOutputProfileData();
 }
-bool gfxPlatformGtk::UseWaylandDMABufWebGL() {
-  return IsWaylandDisplay() && nsWaylandDisplay::IsDMABufWebGLEnabled();
-}
-bool gfxPlatformGtk::UseWaylandHardwareVideoDecoding() {
-  return IsWaylandDisplay() && nsWaylandDisplay::IsDMABufVAAPIEnabled() &&
-         gfxPlatform::CanUseHardwareVideoDecoding();
-}
-#endif
