@@ -7,6 +7,7 @@
 #ifndef MOZILLA_LAYERS_WEBRENDERAPI_H
 #define MOZILLA_LAYERS_WEBRENDERAPI_H
 
+#include <stdint.h>
 #include <vector>
 #include <unordered_map>
 #include <unordered_set>
@@ -16,14 +17,17 @@
 #include "mozilla/layers/IpcResourceUpdateQueue.h"
 #include "mozilla/layers/ScrollableLayerGuid.h"
 #include "mozilla/layers/SyncObject.h"
-#include "mozilla/layers/WebRenderCompositionRecorder.h"
+#include "mozilla/layers/CompositionRecorder.h"
+#include "mozilla/MozPromise.h"
 #include "mozilla/Range.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/webrender/webrender_ffi.h"
 #include "mozilla/webrender/WebRenderTypes.h"
+#include "nsString.h"
 #include "GLTypes.h"
 #include "Units.h"
 
+class gfxContext;
 class nsDisplayItem;
 class nsPaintedDisplayItem;
 class nsDisplayTransform;
@@ -43,6 +47,7 @@ class CompositorBridgeParent;
 class DisplayItemCache;
 class WebRenderBridgeParent;
 class RenderRootStateManager;
+class StackingContextHelper;
 struct DisplayListData;
 }  // namespace layers
 
@@ -55,6 +60,7 @@ namespace wr {
 class DisplayListBuilder;
 class RendererOGL;
 class RendererEvent;
+class WebRenderAPI;
 
 // This isn't part of WR's API, but we define it here to simplify layout's
 // logic and data plumbing.
@@ -88,7 +94,8 @@ struct WrHitResult {
 
 class TransactionBuilder final {
  public:
-  explicit TransactionBuilder(bool aUseSceneBuilderThread = true);
+  explicit TransactionBuilder(WebRenderAPI* aApi,
+                              bool aUseSceneBuilderThread = true);
 
   ~TransactionBuilder();
 
@@ -103,13 +110,12 @@ class TransactionBuilder final {
   void SetDisplayList(const gfx::DeviceColor& aBgColor, Epoch aEpoch,
                       const wr::LayoutSize& aViewportSize,
                       wr::WrPipelineId pipeline_id,
-                      const wr::LayoutSize& content_size,
                       wr::BuiltDisplayListDescriptor dl_descriptor,
                       wr::Vec<uint8_t>& dl_data);
 
   void ClearDisplayList(Epoch aEpoch, wr::WrPipelineId aPipeline);
 
-  void GenerateFrame();
+  void GenerateFrame(const VsyncId& aVsyncId);
 
   void InvalidateRenderedFrame();
 
@@ -196,10 +202,12 @@ class TransactionBuilder final {
   void Clear();
 
   bool UseSceneBuilderThread() const { return mUseSceneBuilderThread; }
+  layers::WebRenderBackend GetBackendType() { return mApiBackend; }
   Transaction* Raw() { return mTxn; }
 
  protected:
   bool mUseSceneBuilderThread;
+  layers::WebRenderBackend mApiBackend;
   Transaction* mTxn;
 };
 
@@ -207,13 +215,16 @@ class TransactionWrapper final {
  public:
   explicit TransactionWrapper(Transaction* aTxn);
 
+  void UpdateDynamicProperties(
+      const nsTArray<wr::WrOpacityProperty>& aOpacityArray,
+      const nsTArray<wr::WrTransformProperty>& aTransformArray,
+      const nsTArray<wr::WrColorProperty>& aColorArray);
   void AppendTransformProperties(
       const nsTArray<wr::WrTransformProperty>& aTransformArray);
   void UpdateScrollPosition(
       const wr::WrPipelineId& aPipelineId,
       const layers::ScrollableLayerGuid::ViewID& aScrollId,
       const wr::LayoutPoint& aScrollPosition);
-  void UpdatePinchZoom(float aZoom);
   void UpdateIsTransformAsyncZooming(uint64_t aAnimationId, bool aIsZooming);
 
  private:
@@ -228,10 +239,13 @@ class WebRenderAPI final {
   static already_AddRefed<WebRenderAPI> Create(
       layers::CompositorBridgeParent* aBridge,
       RefPtr<widget::CompositorWidget>&& aWidget,
-      const wr::WrWindowId& aWindowId, LayoutDeviceIntSize aSize);
+      const wr::WrWindowId& aWindowId, LayoutDeviceIntSize aSize,
+      layers::WindowKind aWindowKind, nsACString& aError);
 
   already_AddRefed<WebRenderAPI> Clone();
 
+  void DestroyRenderer();
+
   wr::WindowId GetId() const { return mId; }
 
   /// Do a non-blocking hit-testing query on a shared version of the hit
@@ -246,13 +260,16 @@ class WebRenderAPI final {
 
   void Readback(const TimeStamp& aStartTime, gfx::IntSize aSize,
                 const gfx::SurfaceFormat& aFormat,
-                const Range<uint8_t>& aBuffer);
+                const Range<uint8_t>& aBuffer, bool* aNeedsYFlip);
 
   void ClearAllCaches();
   void EnableNativeCompositor(bool aEnable);
   void EnableMultithreading(bool aEnable);
   void SetBatchingLookback(uint32_t aCount);
 
+  void SetClearColor(const gfx::DeviceColor& aColor);
+  void SetProfilerUI(const nsCString& aUIString);
+
   void Pause();
   bool Resume();
 
@@ -263,32 +280,38 @@ class WebRenderAPI final {
   void AccumulateMemoryReport(wr::MemoryReport*);
 
   wr::WrIdNamespace GetNamespace();
+  layers::WebRenderBackend GetBackendType() { return mBackend; }
+  layers::WebRenderCompositor GetCompositorType() { return mCompositor; }
   uint32_t GetMaxTextureSize() const { return mMaxTextureSize; }
   bool GetUseANGLE() const { return mUseANGLE; }
   bool GetUseDComp() const { return mUseDComp; }
   bool GetUseTripleBuffering() const { return mUseTripleBuffering; }
+  bool SupportsExternalBufferTextures() const {
+    return mSupportsExternalBufferTextures;
+  }
   layers::SyncHandle GetSyncHandle() const { return mSyncHandle; }
 
   void Capture();
 
-  void ToggleCaptureSequence();
+  void StartCaptureSequence(const nsCString& aPath, uint32_t aFlags);
+  void StopCaptureSequence();
 
-  void SetCompositionRecorder(
-      UniquePtr<layers::WebRenderCompositionRecorder> aRecorder);
+  void BeginRecording(const TimeStamp& aRecordingStart,
+                      wr::PipelineId aRootPipelineId);
 
   typedef MozPromise<bool, nsresult, true> WriteCollectedFramesPromise;
   typedef MozPromise<layers::CollectedFrames, nsresult, true>
       GetCollectedFramesPromise;
 
   /**
-   * Write the frames collected by the |WebRenderCompositionRecorder| to disk.
+   * Write the frames collected since the call to BeginRecording() to disk.
    *
    * If there is not currently a recorder, this is a no-op.
    */
   RefPtr<WriteCollectedFramesPromise> WriteCollectedFrames();
 
   /**
-   * Return the frames collected by the |WebRenderCompositionRecorder| encoded
+   * Return the frames collected since the call to BeginRecording() encoded
    * as data URIs.
    *
    * If there is not currently a recorder, this is a no-op and the promise will
@@ -298,8 +321,11 @@ class WebRenderAPI final {
 
  protected:
   WebRenderAPI(wr::DocumentHandle* aHandle, wr::WindowId aId,
+               layers::WebRenderBackend aBackend,
+               layers::WebRenderCompositor aCompositor,
                uint32_t aMaxTextureSize, bool aUseANGLE, bool aUseDComp,
-               bool aUseTripleBuffering, layers::SyncHandle aSyncHandle);
+               bool aUseTripleBuffering, bool aSupportsExternalBufferTextures,
+               layers::SyncHandle aSyncHandle);
 
   ~WebRenderAPI();
   // Should be used only for shutdown handling
@@ -309,12 +335,16 @@ class WebRenderAPI final {
 
   wr::DocumentHandle* mDocHandle;
   wr::WindowId mId;
+  layers::WebRenderBackend mBackend;
+  layers::WebRenderCompositor mCompositor;
   int32_t mMaxTextureSize;
   bool mUseANGLE;
   bool mUseDComp;
   bool mUseTripleBuffering;
+  bool mSupportsExternalBufferTextures;
   bool mCaptureSequence;
   layers::SyncHandle mSyncHandle;
+  bool mRendererDestroyed;
 
   // We maintain alive the root api to know when to shut the render backend
   // down, and the root api for the document to know when to delete the
@@ -341,7 +371,10 @@ class WebRenderAPI final {
 class MOZ_RAII AutoTransactionSender {
  public:
   AutoTransactionSender(WebRenderAPI* aApi, TransactionBuilder* aTxn)
-      : mApi(aApi), mTxn(aTxn) {}
+      : mApi(aApi), mTxn(aTxn) {
+    MOZ_RELEASE_ASSERT(mApi);
+    MOZ_RELEASE_ASSERT(aTxn);
+  }
 
   ~AutoTransactionSender() { mApi->SendTransaction(*mTxn); }
 
@@ -359,8 +392,11 @@ struct MOZ_STACK_CLASS StackingContextParams : public WrStackingContextParams {
             WrStackingContextClip::None(),
             nullptr,
             nullptr,
+            nullptr,
             wr::TransformStyle::Flat,
             wr::WrReferenceFrameKind::Transform,
+            false,
+            false,
             nullptr,
             /* prim_flags = */ wr::PrimitiveFlags::IS_BACKFACE_VISIBLE,
             wr::MixBlendMode::Normal,
@@ -392,9 +428,10 @@ struct MOZ_STACK_CLASS StackingContextParams : public WrStackingContextParams {
 /// WebRenderFrameBuilder instead, so the interface may change a bit.
 class DisplayListBuilder final {
  public:
-  DisplayListBuilder(wr::PipelineId aId, const wr::LayoutSize& aContentSize,
-                     size_t aCapacity = 0,
-                     layers::DisplayItemCache* aCache = nullptr);
+  explicit DisplayListBuilder(wr::PipelineId aId,
+                              layers::WebRenderBackend aBackend,
+                              size_t aCapacity = 0,
+                              layers::DisplayItemCache* aCache = nullptr);
   DisplayListBuilder(DisplayListBuilder&&) = default;
 
   ~DisplayListBuilder();
@@ -407,8 +444,7 @@ class DisplayListBuilder final {
              const Maybe<usize>& aEnd);
   void DumpSerializedDisplayList();
 
-  void Finalize(wr::LayoutSize& aOutContentSizes,
-                wr::BuiltDisplayList& aOutDisplayList);
+  void Finalize(wr::BuiltDisplayList& aOutDisplayList);
   void Finalize(layers::DisplayListData& aOutTransaction);
 
   Maybe<wr::WrSpatialId> PushStackingContext(
@@ -419,13 +455,13 @@ class DisplayListBuilder final {
   wr::WrClipChainId DefineClipChain(const nsTArray<wr::WrClipId>& aClips,
                                     bool aParentWithCurrentChain = false);
 
-  wr::WrClipId DefineClip(
-      const Maybe<wr::WrSpaceAndClip>& aParent, const wr::LayoutRect& aClipRect,
-      const nsTArray<wr::ComplexClipRegion>* aComplex = nullptr);
-
-  wr::WrClipId DefineImageMaskClip(const wr::ImageMask& aMask);
-  wr::WrClipId DefineRoundedRectClip(const wr::ComplexClipRegion& aComplex);
-  wr::WrClipId DefineRectClip(wr::LayoutRect aClipRect);
+  wr::WrClipId DefineImageMaskClip(const wr::ImageMask& aMask,
+                                   const nsTArray<wr::LayoutPoint>&,
+                                   wr::FillRule);
+  wr::WrClipId DefineRoundedRectClip(Maybe<wr::WrSpatialId> aSpace,
+                                     const wr::ComplexClipRegion& aComplex);
+  wr::WrClipId DefineRectClip(Maybe<wr::WrSpatialId> aSpace,
+                              wr::LayoutRect aClipRect);
 
   wr::WrSpatialId DefineStickyFrame(const wr::LayoutRect& aContentRect,
                                     const float* aTopMargin,
@@ -454,10 +490,10 @@ class DisplayListBuilder final {
                        const wr::LayoutRect& aClip, bool aIsBackfaceVisible,
                        const wr::ColorF& aColor);
   void PushHitTest(const wr::LayoutRect& aBounds, const wr::LayoutRect& aClip,
-                   bool aIsBackfaceVisible);
+                   bool aIsBackfaceVisible,
+                   const layers::ScrollableLayerGuid::ViewID& aScrollId,
+                   gfx::CompositorHitTestInfo aHitInfo, SideBits aSideBits);
   void PushClearRect(const wr::LayoutRect& aBounds);
-  void PushClearRectWithComplexRegion(const wr::LayoutRect& aBounds,
-                                      const wr::ComplexClipRegion& aRegion);
 
   void PushBackdropFilter(const wr::LayoutRect& aBounds,
                           const wr::ComplexClipRegion& aRegion,
@@ -495,7 +531,8 @@ class DisplayListBuilder final {
                  bool aIsBackfaceVisible, wr::ImageRendering aFilter,
                  wr::ImageKey aImage, bool aPremultipliedAlpha = true,
                  const wr::ColorF& aColor = wr::ColorF{1.0f, 1.0f, 1.0f, 1.0f},
-                 bool aPreferCompositorSurface = false);
+                 bool aPreferCompositorSurface = false,
+                 bool aSupportsExternalCompositing = false);
 
   void PushRepeatingImage(
       const wr::LayoutRect& aBounds, const wr::LayoutRect& aClip,
@@ -510,21 +547,24 @@ class DisplayListBuilder final {
       wr::ImageKey aImageChannel1, wr::ImageKey aImageChannel2,
       wr::WrColorDepth aColorDepth, wr::WrYuvColorSpace aColorSpace,
       wr::WrColorRange aColorRange, wr::ImageRendering aFilter,
-      bool aPreferCompositorSurface = false);
+      bool aPreferCompositorSurface = false,
+      bool aSupportsExternalCompositing = false);
 
   void PushNV12Image(const wr::LayoutRect& aBounds, const wr::LayoutRect& aClip,
                      bool aIsBackfaceVisible, wr::ImageKey aImageChannel0,
                      wr::ImageKey aImageChannel1, wr::WrColorDepth aColorDepth,
                      wr::WrYuvColorSpace aColorSpace,
                      wr::WrColorRange aColorRange, wr::ImageRendering aFilter,
-                     bool aPreferCompositorSurface = false);
+                     bool aPreferCompositorSurface = false,
+                     bool aSupportsExternalCompositing = false);
 
   void PushYCbCrInterleavedImage(
       const wr::LayoutRect& aBounds, const wr::LayoutRect& aClip,
       bool aIsBackfaceVisible, wr::ImageKey aImageChannel0,
       wr::WrColorDepth aColorDepth, wr::WrYuvColorSpace aColorSpace,
       wr::WrColorRange aColorRange, wr::ImageRendering aFilter,
-      bool aPreferCompositorSurface = false);
+      bool aPreferCompositorSurface = false,
+      bool aSupportsExternalCompositing = false);
 
   void PushIFrame(const wr::LayoutRect& aBounds, bool aIsBackfaceVisible,
                   wr::PipelineId aPipeline, bool aIgnoreMissingPipeline);
@@ -623,6 +663,7 @@ class DisplayListBuilder final {
   }
 
   const wr::PipelineId& CurrentPipelineId() const { return mPipelineId; }
+  layers::WebRenderBackend GetBackendType() const { return mBackend; }
 
   // Checks to see if the innermost enclosing fixed pos item has the same
   // ASR. If so, it returns the scroll target for that fixed-pos item.
@@ -632,13 +673,6 @@ class DisplayListBuilder final {
 
   Maybe<SideBits> GetContainingFixedPosSideBits(const ActiveScrolledRoot* aAsr);
 
-  // Set the hit-test info to be used for all display items until the next call
-  // to SetHitTestInfo or ClearHitTestInfo.
-  void SetHitTestInfo(const layers::ScrollableLayerGuid::ViewID& aScrollId,
-                      gfx::CompositorHitTestInfo aHitInfo, SideBits aSideBits);
-  // Clears the hit-test info so that subsequent display items will not have it.
-  void ClearHitTestInfo();
-
   already_AddRefed<gfxContext> GetTextContext(
       wr::IpcResourceUpdateQueue& aResources,
       const layers::StackingContextHelper& aSc,
@@ -712,6 +746,7 @@ class DisplayListBuilder final {
   FixedPosScrollTargetTracker* mActiveFixedPosTracker;
 
   wr::PipelineId mPipelineId;
+  layers::WebRenderBackend mBackend;
   wr::LayoutSize mContentSize;
 
   nsTArray<wr::PipelineId> mRemotePipelineIds;
@@ -728,28 +763,22 @@ class DisplayListBuilder final {
 class MOZ_RAII SpaceAndClipChainHelper final {
  public:
   SpaceAndClipChainHelper(DisplayListBuilder& aBuilder,
-                          wr::WrSpaceAndClipChain aSpaceAndClipChain
-                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+                          wr::WrSpaceAndClipChain aSpaceAndClipChain)
       : mBuilder(aBuilder),
         mOldSpaceAndClipChain(aBuilder.mCurrentSpaceAndClipChain) {
     aBuilder.mCurrentSpaceAndClipChain = aSpaceAndClipChain;
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
   }
   SpaceAndClipChainHelper(DisplayListBuilder& aBuilder,
-                          wr::WrSpatialId aSpatialId
-                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+                          wr::WrSpatialId aSpatialId)
       : mBuilder(aBuilder),
         mOldSpaceAndClipChain(aBuilder.mCurrentSpaceAndClipChain) {
     aBuilder.mCurrentSpaceAndClipChain.space = aSpatialId;
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
   }
   SpaceAndClipChainHelper(DisplayListBuilder& aBuilder,
-                          wr::WrClipChainId aClipChainId
-                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+                          wr::WrClipChainId aClipChainId)
       : mBuilder(aBuilder),
         mOldSpaceAndClipChain(aBuilder.mCurrentSpaceAndClipChain) {
     aBuilder.mCurrentSpaceAndClipChain.clip_chain = aClipChainId.id;
-    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
   }
 
   ~SpaceAndClipChainHelper() {
@@ -761,7 +790,6 @@ class MOZ_RAII SpaceAndClipChainHelper final {
 
   DisplayListBuilder& mBuilder;
   wr::WrSpaceAndClipChain mOldSpaceAndClipChain;
-  MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 };
 
 Maybe<wr::ImageFormat> SurfaceFormatToImageFormat(gfx::SurfaceFormat aFormat);
